/*
 * Downloader.java
 * 17. November 2007, 23:15
 */

package de.siedler_maps.mapinstaller;

import java.io.*;
import java.net.URL;

/**
 * Runnable class to download a file from a webserver.
 *
 * @author nevermind
 **/
public class Downloader
extends Thread
{
    /** Callback handler */
    DownloadCallback callback;
    /** URL of the source file to be downloaded. */
    URL source;
    /** Destination of the downloaded file in the local filesystem. */
    File destination;
    
    /** State flag of the thread. */
    private boolean canceled = false;
    
    /** Creates a new downloader. */
    public Downloader(URL source, File destination, DownloadCallback callback)
    {
        this.source         = source;
        this.destination    = destination;
        this.callback       = callback;
    }

    /** Returns the destionation File object. */
    public File getDestination()
    {
        return destination;
    }

    /** Returns the source URL object. */
    public URL getSource()
    {
        return source;
    }
    
    /** Cancel the download. */
    public void cancel()
    {
        canceled = true;
    }

    public void run()
    {
        boolean success = true;
        OutputStream out = null;
        InputStream in = null;
        
        try
        {
            out = new BufferedOutputStream(new FileOutputStream(destination));
            in = source.openStream();

            // Start download
            byte[] buffer = new byte[1024];
            int bytesRead, totalBytesRead = 0;
            while ((bytesRead = in.read(buffer)) != -1)
            {
                out.write(buffer, 0, bytesRead);
                totalBytesRead += bytesRead;

                // Callback
                callback.downloadUpdate(totalBytesRead);
                
                if (canceled)
                {
                    throw new IOException("Download incomplete");
                }
            }
        }
        catch (IOException ex)
        {
            success = false;
        }
        // Close open streams
        finally
        {
            try
            {
                if (in != null)
                {
                    in.close(); 
                }
                if (out != null)
                {
                    out.close();
                }
            }
            catch (IOException ex)
            {
            }
        }
        
        // If download failed, delete the partial downloaded file
        if (!success)
        {
            destination.delete();
        }
        
        callback.downloadFinished(success);
    }
}
