/*
 * SpeedMeter.java
 * 18. November 2007, 11:23
 */

package de.siedler_maps.mapinstaller;

import java.awt.event.ActionListener;
import javax.swing.Timer;

/**
 * A class to calculate the average download speed.
 *
 * @author nevermind
 **/
public class SpeedMeter
extends Timer
{
    /** Size of the speed history. */
    private final int periods = 3;
    
    /** Avarage download speed. */
    private int[] averageSpeed;
    /** Downloaded bytes in the last second. */
    private int lastBytesRead = 0;
    
    /** Init a new SpeedMeter. */
    public SpeedMeter(ActionListener callback)
    {
        super(1000, callback);
        setActionCommand("SpeedMeterCallback");
        
        // Init history array
        averageSpeed = new int[periods];
        for (int i = 0; i < averageSpeed.length; i++)
        {
            averageSpeed[i] = 0;
        }
    }
    
    /**
     * Add a new speed period and update average speed.
     *
     * @param bytesRead Number of total bytes read.
     **/
    public void update(int bytesRead)
    {
        // Downloaded bytes in this second
        int speed = bytesRead - lastBytesRead;
        lastBytesRead = bytesRead;
        
        // First time this function is called?
        if (averageSpeed[0] == 0)
        {
            // Init the history with the current number of read bytes
            for (int i = 0; i < averageSpeed.length; i++)
            {
                averageSpeed[i] = speed;
            }
        }
        
        // Update average speed
        for (int i = 0; i < averageSpeed.length; i++)
        {
            speed += averageSpeed[i];
        }
        speed /= averageSpeed.length + 1;
        
        // Update history
        for (int i = averageSpeed.length - 1; i > 0; i--)
        {
            averageSpeed[i] = averageSpeed[i-1];
        }
        averageSpeed[0] = speed;
    }
    
    /**
     * Returns the current download speed.
     *
     * @return download speed in bytes.
     **/
    public int getSpeed()
    {
        return averageSpeed[0];
    }
}
